package com.FranckBarbier.Java._Inheritance_and_genericity;

abstract public class Vector<T extends Number> extends Number {

    abstract protected Vector<T> create(int size);
    protected java.util.Vector<Number> _implementation;

    public Vector(T[] t) {
        _implementation = new java.util.Vector<Number>(t.length);
        for (int i = 0; i < t.length; i++) {
            _implementation.addElement(t[i]);
        }
    }

    public Vector(int size) {
        _implementation = new java.util.Vector<Number>(size);
    }

    protected double accumulate() {
        double result = 0.;
        for (Number n : _implementation) {
            result += n.doubleValue();
        }
        return result;
    }

    protected double max() {
        double result = Double.NEGATIVE_INFINITY;
        for (Number n : _implementation) {
            result = n.doubleValue() > result ? n.doubleValue() : result;
        }
        return result;
    }

    abstract public double norm();

    public double norm_addition(final Vector<T> v) {
        return this.plus(v).norm();
    }

    public double norm_subtraction(final Vector<T> v) {
        return this.minus(v).norm();
    }

    public double lambda_norm(final double lambda) {
        return Math.abs(lambda) * norm();
    }

    public Number get(int index) {
        return _implementation.elementAt(index);
    }

    public void set(int index, Number t) {
        _implementation.insertElementAt(t, index);
    }

    public Vector<T> plus(final Vector<T> v) {
        if (this._implementation.size() == 0 && v._implementation.size() == 0) {
            return create(0);
        }
        /* size harmonization when an operand's size == 0 */
        if (this._implementation.size() != v._implementation.size()) {
            throw new RuntimeException("Size is different");
        }
        Vector<T> result = create(this._implementation.size());
        for (int i = 0; i < this._implementation.size(); i++) {
            try {
                result._implementation.insertElementAt(Vector.class.cast(this._implementation.get(i)).plus(Vector.class.cast(v._implementation.get(i))), i);
            } catch (ClassCastException cce) {
                result._implementation.insertElementAt(_implementation.get(i).doubleValue() + v._implementation.get(i).doubleValue(), i);
            }
        }
        return result;
    }

    public Vector<T> minus(final Vector<T> v) {
        if (this._implementation.size() == 0 && v._implementation.size() == 0) {
            return create(0);
        }
        /* size harmonization when an operand's size == 0 */
        if (this._implementation.size() != v._implementation.size()) {
            throw new RuntimeException("Size is different");
        }
        Vector<T> result = create(this._implementation.size());
        for (int i = 0; i < this._implementation.size(); i++) {
            try {
                result._implementation.insertElementAt(Vector.class.cast(this._implementation.get(i)).minus(Vector.class.cast(v._implementation.get(i))), i);
            } catch (ClassCastException cce) {
                result._implementation.insertElementAt(_implementation.get(i).doubleValue() - v._implementation.get(i).doubleValue(), i);
            }
        }
        return result;
    }

    public Vector<T> multiply(final Vector<T> v) {
        if (this._implementation.size() == 0 && v._implementation.size() == 0) {
            return create(0);
        }
        /* size harmonization when an operand's size == 0 */
        if (this._implementation.size() != v._implementation.size()) {
            throw new RuntimeException("Size is different");
        }
        Vector<T> result = create(this._implementation.size());
        for (int i = 0; i < this._implementation.size(); i++) {
            try {
                result._implementation.insertElementAt(Vector.class.cast(this._implementation.get(i)).multiply(Vector.class.cast(v._implementation.get(i))), i);
            } catch (ClassCastException cce) {
                result._implementation.insertElementAt(_implementation.get(i).doubleValue() * v._implementation.get(i).doubleValue(), i);
            }
        }
        return result;
    }

    public Vector<T> divide(final Vector<T> v) {
        if (this._implementation.size() == 0 && v._implementation.size() == 0) {
            return create(0);
        }
        /* size harmonization when an operand's size == 0 */
        if (this._implementation.size() != v._implementation.size()) {
            throw new RuntimeException("Size is different");
        }
        Vector<T> result = create(this._implementation.size());
        for (int i = 0; i < this._implementation.size(); i++) {
            try {
                result._implementation.insertElementAt(Vector.class.cast(this._implementation.get(i)).divide(Vector.class.cast(v._implementation.get(i))), i);
            } catch (ClassCastException cce) {
                result._implementation.insertElementAt(_implementation.get(i).doubleValue() / v._implementation.get(i).doubleValue(), i);
            }
        }
        return result;
    }
}
